classdef Request
    %COMMANDREQUEST Summary of this class goes here
    %   Detailed explanation goes here
    
    properties (Constant)
       MAGIC_TOKEN = uint32(hex2dec('53504953'));
       header_size = 32; % Command header size
    end
    
    properties (SetAccess = private)
        magic_token;
        command_id;
        command;
        padding = typecast(int32(0), 'uint8');
        data_size;
        reserved = typecast(int64(0), 'uint8');
        cmdobj;
    end
    
    methods
        function obj = Request(command)
            %COMMAND Construct an instance of this class
            %   Detailed explanation goes here
            persistent req_count;
            
            if nargin == 1
                if isempty(req_count); req_count = uint32(0); end
                req_count = uint32(mod(req_count+1, intmax('uint32')));
                
                cmd_size = command.size();
                pad_size = 32 - mod(cmd_size, 32); % Pad command data to multiple of 32 bytes
                
                obj.magic_token = obj.MAGIC_TOKEN;
                obj.command_id = req_count;
                obj.command = command.type;
                obj.data_size = cmd_size + pad_size;
                obj.cmdobj = command;
            end
        end
        
        function N = size(obj)
           N = obj.header_size+obj.data_size; 
        end
        
        function bytes = serialize(obj)
            %METHOD1 Summary of this method goes here
            %   Detailed explanation goes here
            
            bytes = zeros(1, obj.header_size+obj.data_size, 'uint8'); % allocate
            
            % CommandHeader
            bytes(1:4) = typecast(obj.magic_token, 'uint8'); % magic_token
            bytes(5:8) = typecast(uint32(obj.command_id), 'uint8'); % command_id
            bytes(9:12) = typecast(uint32(obj.command), 'uint8'); % command
            % bytes(13:16) = uint8(0); % padding
            bytes(17:24) = typecast(uint64(obj.data_size), 'uint8'); % data size
            % bates(25:32) = uint8(0); % reserved
            
            n_bytes = obj.cmdobj.size;
            bytes(32+(1:n_bytes)) = obj.cmdobj.serialize();
        end
    end
    
    methods (Static)
        function obj = deserialize(bytes)
            if ~isa(bytes, 'uint8')
                error('Input parameter must be an array of type uint8.');
            end
            
            msg_size = numel(bytes);
            
            if msg_size < PulseStreamer.internal.Request.header_size
                error('Message is shorter than the header length');
            end
            
            obj = PulseStreamer.internal.Request();
            obj.magic_token = typecast(bytes(1:4), 'uint32');
            
            if obj.magic_token ~= obj.MAGIC_TOKEN
                error('Magic token is incorrect. Message data is damaged.');
            end
            obj.command_id = typecast(bytes(5:8), 'uint32');
            obj.command = PulseStreamer.internal.CommandEnum(typecast(bytes(9:12), 'uint32'));
            obj.data_size = typecast(bytes(17:24), 'uint64');
            
            if (obj.data_size+obj.header_size) ~= msg_size 
                error('Message has inconsistent number of bytes')
            end
            
            data_bytes = bytes((obj.header_size+1):end);
            
            switch obj.commmand
                case PulseStreamer.internal.CommandEnum.StreamHighLevel
                    obj.cmdobj = PulseStreamer.internal.cmdStreamHighLevel.deserialize(data_bytes);
                    
                otherwise
                    error('Unknown command type');
            end
        end
    end
end

