classdef Response
    %COMMANDRESPONSE Summary of this class goes here
    %   Detailed explanation goes here
    
    properties (Constant)
        MAGIC_TOKEN = uint32(hex2dec('53504953'));
        header_size = 32; % Command header size
    end
    
    properties (SetAccess = private, GetAccess = public)
        magic_token
        command_id
        status
        reserved1
        size
        reserved2
    end
    
    methods
        function obj = Response(bytes)
            %COMMANDRESPONSE Parses binary protocol command response bytes.
            if ~isa(bytes, 'uint8')
                error('Input parameter must be an array of type uint8.');
            end
            
            if numel(bytes) ~= PulseStreamer.internal.Response.header_size
                obj.status = PulseStreamer.internal.StatusEnum.ClientParseError;
                return
            end
            
            % Parse values
            obj.magic_token = typecast(bytes(1:4), 'uint32');
            obj.command_id = typecast(bytes(5:8), 'uint32');
            obj.status = PulseStreamer.internal.StatusEnum(typecast(bytes(9:12), 'uint32'));
            obj.reserved1 = bytes(13:16);
            obj.size = typecast(bytes(17:24), 'uint64');
            obj.reserved2 = bytes(25:32);
            
            if obj.magic_token ~= PulseStreamer.internal.Response.MAGIC_TOKEN
                % Magic token is incorrect
                obj.status = PulseStreamer.internal.StatusEnum.ClientParseError;
                warning('Magic token is incorrect. Response data is corrupt.')
            end
        end
        
        function bytes = serialize(obj)
           
            bytes = uint8(zeros(1,obj.header_size)); % allocate
            bytes(1:4) = typecast(uint32(obj.magic_token), 'uint8');
            bytes(5:8) = typecast(uint32(obj.command_id), 'uint8'); 
            bytes(9:12) = typecast(uint32(obj.status), 'uint8'); 
            bytes(13:16) = uint8(obj.reserved1);
            bytes(17:24) = typecast(uint64(obj.size), 'uint8');
            bytes(25:32) = uint8(obj.reserved2);
        end
        
    end
end

