classdef cmdStreamHighLevel
    %CMDSTREAMHIGHLEVEL Summary of this class goes here
    %   Detailed explanation goes here
    
    properties (Constant)
        type = PulseStreamer.internal.CommandEnum.StreamHighLevel;
        header_size = 32;
    end
    
    properties (SetAccess = private)
        seq_bytes = uint8([]);
        n_runs = int64(0);
        n_pulses = int64(0);
        final_analog = int16([0,0]);
        final_digital = uint8(0);
        padding = uint8([0,0,0]);
        reserved = typecast(uint64(0), 'uint8');
    end
    
    methods
        function obj = cmdStreamHighLevel(seq_bytes, n_runs, n_pulses, final_analog, final_digital)
            %CMDSTREAMHIGHLEVEL Construct an instance of this class
            %   Detailed explanation goes here
            if nargin == 0
                return
            end
            
            obj.seq_bytes = seq_bytes;
            obj.n_runs = n_runs;
            obj.n_pulses = n_pulses;
            obj.final_analog = final_analog;
            obj.final_digital = final_digital;
        end
        
        function N = size(obj)
            % Size of this object's data and header in bytes
            N = obj.header_size+numel(obj.seq_bytes);
        end
        
        function bytes = serialize(obj)
            %METHOD1 Summary of this method goes here
            %   Detailed explanation goes here
            
            bytes = zeros(1, obj.size(), 'uint8'); % allocate
            
            % StreamHighLevelHeader
            bytes(1:8) = typecast(int64(obj.n_runs), 'uint8'); % n_runs
            bytes(9:16) = typecast(int64(obj.n_pulses), 'uint8'); % n_pulses
            bytes(17:20) = typecast(int16(obj.final_analog), 'uint8'); % final_analog
            bytes(21) = uint8(obj.final_digital); % final_digital
            % bytes(22:24) = uint8(0); % padding
            % bytes(25:32) = uint8(0); % reserved
            
            n_bytes = numel(obj.seq_bytes);
            bytes(32+(1:n_bytes)) = obj.seq_bytes;
        end
    end
    
    
    methods (Static)
        function obj = deserialize(bytes)
            
            if ~isa(bytes, 'uint8')
                error('Input parameter must be an array of type uint8.');
            end
            
            msg_size = numel(bytes);
            
            if msg_size < 32
                error('Message is shorter than minimal length');
            end
            
            obj = PulseStreamer.internal.cmdStreamHighLevel();
            
            obj.n_runs = typecast(bytes(1:8), 'int64');
            obj.n_pulses = typecast(bytes(9:16), 'int64');
            obj.final_analog = typecast(bytes(17:20), 'int16');
            obj.final_digital = typecast(bytes(21), 'uint8');
            
            seq_bytes = bytes((obj.header_size+1):end);
            seq_size = floor(numel(seq_bytes)/9)*9;
            
            if seq_size == 0
                warning('Sequence is empty');
            end
            
            obj.seq_bytes = bytes(obj.header_size+(1:seq_size));
            
        end
    end
end

